package com.yc.ship.module.resource.controller.admin.room;

import com.yc.ship.framework.common.util.collection.CollectionUtils;
import com.yc.ship.module.resource.dal.dataobject.roommodel.ResourceRoomModelDO;
import com.yc.ship.module.resource.dal.dataobject.ship.ResourceShipDO;
import com.yc.ship.module.resource.service.roommodel.ResourceRoomModelService;
import com.yc.ship.module.resource.service.ship.ResourceShipService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import com.yc.ship.framework.common.pojo.PageParam;
import com.yc.ship.framework.common.pojo.PageResult;
import com.yc.ship.framework.common.pojo.CommonResult;
import com.yc.ship.framework.common.util.object.BeanUtils;
import static com.yc.ship.framework.common.pojo.CommonResult.success;

import com.yc.ship.framework.excel.core.util.ExcelUtils;

import com.yc.ship.framework.apilog.core.annotation.ApiAccessLog;
import static com.yc.ship.framework.apilog.core.enums.OperateTypeEnum.*;

import com.yc.ship.module.resource.controller.admin.room.vo.*;
import com.yc.ship.module.resource.dal.dataobject.room.ResourceRoomDO;
import com.yc.ship.module.resource.service.room.ResourceRoomService;

@Tag(name = "管理后台 - 资源管理-游轮房间管理")
@RestController
@RequestMapping("/resource/room")
@Validated
public class ResourceRoomController {

    @Resource
    private ResourceRoomService roomService;

    @Resource
    private ResourceShipService shipService;

    @Resource
    private ResourceRoomModelService modelService;

    @PostMapping("/create")
    @Operation(summary = "创建资源管理-游轮房间管理")
    @PreAuthorize("@ss.hasPermission('resource:room:create')")
    public CommonResult<Long> createRoom(@Valid @RequestBody ResourceRoomSaveReqVO createReqVO) {
        return success(roomService.createRoom(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新资源管理-游轮房间管理")
    @PreAuthorize("@ss.hasPermission('resource:room:update')")
    public CommonResult<Boolean> updateRoom(@Valid @RequestBody ResourceRoomSaveReqVO updateReqVO) {
        roomService.updateRoom(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除资源管理-游轮房间管理")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('resource:room:delete')")
    public CommonResult<Boolean> deleteRoom(@RequestParam("id") Long id) {
        roomService.deleteRoom(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得资源管理-游轮房间管理")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('resource:room:query')")
    public CommonResult<ResourceRoomRespVO> getRoom(@RequestParam("id") Long id) {
        ResourceRoomDO room = roomService.getRoom(id);
        return success(BeanUtils.toBean(room, ResourceRoomRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得资源管理-游轮房间管理分页")
    @PreAuthorize("@ss.hasPermission('resource:room:query')")
    public CommonResult<PageResult<ResourceRoomRespVO>> getRoomPage(@Valid ResourceRoomPageReqVO pageReqVO) {
        PageResult<ResourceRoomDO> pageResult = roomService.getRoomPage(pageReqVO);
        PageResult<ResourceRoomRespVO> page = BeanUtils.toBean(pageResult, ResourceRoomRespVO.class);
        List<ResourceRoomRespVO> list = page.getList();
        //处理游轮名称
        List<Long> shipIds = CollectionUtils.convertList(list, ResourceRoomRespVO::getShipId);
        List<ResourceShipDO> shipList = shipService.getList(shipIds);
        Map<Long, ResourceShipDO> longResourceShipDOMap = CollectionUtils.convertMap(shipList, ResourceShipDO::getId);

        //处理房型名称
        List<Long> modelIds = CollectionUtils.convertList(list, ResourceRoomRespVO::getRoomModelId);
        List<ResourceRoomModelDO> modelList = modelService.getList(modelIds);
        Map<Long, ResourceRoomModelDO> longResourceRoomModelDOMap = CollectionUtils.convertMap(modelList, ResourceRoomModelDO::getId);
        list.stream().forEach(item -> {
            item.setShipName(longResourceShipDOMap.get(item.getShipId()).getName());
            item.setRoomModelName(longResourceRoomModelDOMap.get(item.getRoomModelId()).getName());
        });
        return success(page);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出资源管理-游轮房间管理 Excel")
    @PreAuthorize("@ss.hasPermission('resource:room:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportRoomExcel(@Valid ResourceRoomPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<ResourceRoomDO> list = roomService.getRoomPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "资源管理-游轮房间管理.xls", "数据", ResourceRoomRespVO.class,
                        BeanUtils.toBean(list, ResourceRoomRespVO.class));
    }

    @GetMapping("/getRoomListByShipId")
    @Operation(summary = "根据游轮ID获取游轮房间列表")
    public CommonResult<List<ResourceRoomRespVO>> getRoomListByShipId(@RequestParam("shipId") String shipId) {
        List<ResourceRoomDO> list = roomService.getRoomListByShipId(shipId);
        return success(BeanUtils.toBean(list, ResourceRoomRespVO.class));
    }

    @GetMapping("/getRoomListByShipIdAndFloors")
    @Operation(summary = "根据游轮ID获取游轮房间管理")
    public CommonResult<List<ResourceRoomRespVO>> getRoomListByShipIdAndFloors(@RequestParam("shipId") String shipId, @RequestParam("floors") String floors) {
        List<ResourceRoomDO> list = roomService.getRoomListByShipIdAndFloors(shipId, floors);
        return success(BeanUtils.toBean(list, ResourceRoomRespVO.class));
    }

}